/*

    .oooo.     .oooo.        88       .o        88  .ooooo.    ooooooooo
  .dP""Y88b  .dP""Y88b      .8'     .d88       .8' 888' `Y88. d"""""""8'
	]8P'       ]8P'    .8'    .d'888      .8'  888    888       .8'
      .d8P'      <88b.    .8'   .d'  888     .8'    `Vbood888      .8'
    .dP'          `88b.  .8'    88ooo888oo  .8'          888'     .8'
  .oP     .o o.   .88P  .8'          888   .8'         .88P'     .8'
  8888888888 `8bd88P'   88          o888o  88        .oP'       .8'

     []-------------------------------------------------------------[]
     |		                CIRCUNF.C                            |
     |						 	             |
     |     Determinacao do raio e ponto sobre uma circunferencia     |
     |						                     |
     |           por Lenimar N. Andrade, maio/96-abril/97            |
     []-------------------------------------------------------------[]
*/

#include "vetores.c"

/* ----------------------------------------------------------------------- */

double Angulo(double x, double y) {

/* Retorna o angulo no intervalo [0, 2] que o vetor (x, y) forma
   com (1, 0) */

  double r;

  r = sqrt(x*x + y*y);
  if (r <= z_erro) return 0;
  else if (y >= 0) return acos(x/r);
  else return acos(-x/r) + M_PI;
}

/* ----------------------------------------------------------------------- */

int PontoNaCircunferencia(ponto *R, ponto P, vetor v1, ponto Q, vetor v2,
						     ponto C, double teta) {

/* Determina o ponto  R na circunferencia no espaco R3 de centro C e que
   passa pelo ponto Q e contida no plano paralelo aos vetores v1 e v2. R
   e' definido como sendo o ponto desta circunferencia tal que o arco QR
   mede  teta radianos (a partir de Q, no sentido de P  para Q). Retorna
   0 somente se houver erro. */

  double r, rot[3][3], beta, a, b, c, p, q, t, lambda, V, lambdaV;
  ponto P2, Q2, R2;

  PROD_VET(a, b, c, v1.x, v1.y, v1.z, v2.x, v2.y, v2.z);

  q = a*v1.x + b*v1.y + c*v1.z;
  if (fabs(q) > z_erro) {
    p = a*(Q.x - P.x) + b*(Q.y - P.y) + c*(Q.z - P.z);
    t = p/q;
    P.x += t*v1.x, P.y += t*v1.y, P.z += t*v1.z;
  }

  r = DISTANCIA(C.x, C.y, C.z, Q.x, Q.y, Q.z);  /* raio da circunferencia */

  /* Calculo da matriz de rotacao rot[][] */
  V = NORMA(a, b, c);
  if (V < z_erro) return 0;
  lambda = sqrt(b*b + c*c);
  lambdaV = lambda*V;
  if (lambda > z_erro) {
    rot[0][0] = lambda/V, rot[0][1] = 0, rot[0][2] = a/V;
    rot[1][0] = -a*b/lambdaV, rot[1][1] = c/lambda, rot[1][2] = b/V;
    rot[2][0] = -a*c/lambdaV, rot[2][1] = -b/lambda, rot[2][2] = c/V;
  } else {
    rot[0][0] = 0, rot[0][1] = 0, rot[0][2] = (a >= 0 ? 1 : -1);
    rot[1][0] = 0, rot[1][1] = 1, rot[1][2] = 0;
    rot[2][0] = (a >= 0 ? -1 : 1), rot[2][1] = 0, rot[2][2] = 0;
  }

  /* Rotacao dos pontos P e Q, segundo a matriz rot[][]. P e Q sao
     levados para P2 e Q2, respectivamente. */
  P2.x = (P.x - C.x)*rot[0][0] + (P.y - C.y)*rot[1][0] + (P.z - C.z)*rot[2][0];
  P2.y = (P.x - C.x)*rot[0][1] + (P.y - C.y)*rot[1][1] + (P.z - C.z)*rot[2][1];
  P2.z = (P.x - C.x)*rot[0][2] + (P.y - C.y)*rot[1][2] + (P.z - C.z)*rot[2][2];

  Q2.x = (Q.x - C.x)*rot[0][0] + (Q.y - C.y)*rot[1][0] + (Q.z - C.z)*rot[2][0];
  Q2.y = (Q.x - C.x)*rot[0][1] + (Q.y - C.y)*rot[1][1] + (Q.z - C.z)*rot[2][1];
  Q2.z = (Q.x - C.x)*rot[0][2] + (Q.y - C.y)*rot[1][2] + (Q.z - C.z)*rot[2][2];

  beta = Angulo(Q2.x, Q2.y);

  /* Calculo do ponto R2 */
  if (P2.x*Q2.y - P2.y*Q2.x > 0) {
    R2.x = r*cos(beta + teta);
    R2.y = r*sin(beta + teta);
    R2.z = 0;
  } else {
    R2.x = r*cos(beta - teta);
    R2.y = r*sin(beta - teta);
    R2.z = 0;
  }

  /* Rotacao inversa do ponto R2 */
  R->x = (R2.x)*rot[0][0] + (R2.y)*rot[0][1] + (R2.z)*rot[0][2] + C.x;
  R->y = (R2.x)*rot[1][0] + (R2.y)*rot[1][1] + (R2.z)*rot[1][2] + C.y;
  R->z = (R2.x)*rot[2][0] + (R2.y)*rot[2][1] + (R2.z)*rot[2][2] + C.z;

  return 1;
}

/* ----------------------------------------------------------------------- */

int CentroDaCircunferencia(ponto *C, double *raio, ponto P, vetor v1,
						   ponto Q, vetor v2) {

  /* Determina  o  centro  da  circunferencia  que  passa  pelos pontos P e
     Q,  cujo centro  situa-se na intersecao dos  planos   que  passam  por
     P  e  Q  e  cujos vetores normais sao v1 e v2. Retorna 0
     somente se houver erro. */

  vetor n;

  PROD_VET(n.x, n.y, n.z, v1.x, v1.y, v1.z, v2.x, v2.y, v2.z);

  if (!ResolveSistema3x3(v1.x, v1.y, v1.z, P.x*v1.x + P.y*v1.y + P.z*v1.z,
			 v2.x, v2.y, v2.z, Q.x*v2.x + Q.y*v2.y + Q.z*v2.z,
			 n.x,  n.y,  n.z,  Q.x*n.x  + Q.y*n.y  + Q.z*n.z,
			 &C->x, &C->y, &C->z)) {
			 *raio = C->x = C->y = C->z = INF; return 0;}

  *raio = DISTANCIA(C->x, C->y, C->z, Q.x, Q.y, Q.z);

  return 1;
}

/* ----------------------------------------------------------------------- */

/*** FIM DE CIRCUNF.C ***/
