/*

	     .o        88   oooooooo      88  .ooooo.    ooooooooo
	   .d88       .8'  dP"""""""     .8' 888' `Y88. d"""""""8'
	 .d'888      .8'  d88888b.      .8'  888    888       .8'
       .d'  888     .8'       `Y88b    .8'    `Vbood888      .8'
       88ooo888oo  .8'          ]88   .8'          888'     .8'
	    888   .8'     o.   .88P  .8'         .88P'     .8'
	   o888o  88      `8bd88P'   88        .oP'       .8'

     []-------------------------------------------------------------[]
     |		                 GRAF.C                              |
     |						 	             |
     |          Funcoes para o tracado de graficos 2D e 3D           |
     |						                     |
     |                por Lenimar N. Andrade, maio/97                |
     []-------------------------------------------------------------[]
*/

#include <graphics.h>
#include <conio.h>
#include <string.h>
#include <dos.h>
#include "circunf.c"

/* #define GRAF2D */
/* #define GRAF3D */

#define PI 3.1415926535
#define DOIS_PI 6.283185307
int A, B,  /* Coordenadas da origem na tela     */
    TAM;   /* Constante de ampliacao do grafico */

/* ---------------------------------------------------------------------- */

int IniciaModoGrafico(void) {

  /* Coloca a tela no modo grafico */

  int DriverGrafico, ModoGrafico;
  registerbgidriver(EGAVGA_driver);
  DriverGrafico = DETECT;
  initgraph(&DriverGrafico, &ModoGrafico, "");
  if (graphresult() < 0) {
    printf("\nERRO na mudanca para o modo grafico.\n");
    return 0;
  }
  A = getmaxx()/2, B = getmaxy()/2;
  return 1;
}

/* ---------------------------------------------------------------------- */

#ifdef GRAF2D

void DesenhaGraficoParam(double (*f)(double), double (*g)(double),
		    double tmin, double tmax, int cor) {

/* Desenha o grafico de (f(t), g(t)) com t variando de tmin a tmax */

  double t, inc = 0.02;

  inc = (tmax - tmin)/100.0;
  setcolor(cor);
  for (t = tmin; t <= tmax - inc; t += inc) {
    line((int) (A + f(t)*TAM), (int) (B - g(t)*TAM),
	 (int) (A + f(t + inc)*TAM), (int) (B - g(t + inc)*TAM));
  }
  while (t < tmax) {
    putpixel((int) (A + f(t)*TAM), (int) (B - g(t)*TAM), cor);
    t += 0.001;
  }
}

/* ---------------------------------------------------------------------- */

void MarcaPonto(double X, double Y, int cor) {

/* Desenha o ponto (X, Y) no formato de um "x" */

  int a, b;
  a = (int) (A + X*TAM), b = (int) (B - Y*TAM);
  putpixel(a, b, cor);
  putpixel(a + 1, b + 1, cor);
  putpixel(a + 1, b - 1, cor);
  putpixel(a - 1, b + 1, cor);
  putpixel(a - 1, b - 1, cor);
  putpixel(a + 2, b + 2, cor);
  putpixel(a + 2, b - 2, cor);
  putpixel(a - 2, b + 2, cor);
  putpixel(a - 2, b - 2, cor);
}

/* ---------------------------------------------------------------------- */

double XCentro, YCentro, ZCentro, Raio;

/* Circunferencia de centro (XCentro, YCentro, Zcentro) */

double g1(double t) { return XCentro + Raio*cos(t); }
double g2(double t) { return YCentro + Raio*sin(t); }
double g3(double t) { return ZCentro; }

#endif

/* ---------------------------------------------------------------------- */

#ifdef GRAF3D

double fi = -PI/4, teta = -PI/4;

void ProjetaPonto(double x, double y, double z,
					  double *proj_X, double *proj_Y) {

/* Projeta o ponto (x, y, z) em coordenadas (proj_X, proj_Y) da tela */

  *proj_X = y*cos(teta) + x*sin(teta);
  *proj_Y = z*cos(fi) + x*cos(teta)*sin(fi) - y*sin(fi)*sin(teta);
}

/* ---------------------------------------------------------------------- */

void DesenhaEixos(int tam, int cor1, int cor2) {

/* Desenha os eixos x, y, z usando tamanho e cores especificados */

  double X0, Y0, X, Y;
  int x_x, x_y, y_x, y_y, z_x, z_y;

  setcolor(cor1);

  ProjetaPonto(0, 0, 0, &X0, &Y0);
  ProjetaPonto(tam, 0, 0, &X, &Y);
  line(A + X0*TAM, B - Y0*TAM, A + X*TAM, B - Y*TAM);
  x_x = A + X*TAM; x_y = B - Y*TAM;

  ProjetaPonto(0, tam, 0, &X, &Y);
  line(A + X0*TAM, B - Y0*TAM, A + X*TAM, B - Y*TAM);
  y_x = A + X*TAM; y_y = B - Y*TAM;

  ProjetaPonto(0, 0, tam, &X, &Y);
  line(A + X0*TAM, B - Y0*TAM, A + X*TAM, B - Y*TAM);
  z_x = A + X*TAM; z_y = B - Y*TAM;

  setcolor(cor2);
  outtextxy(x_x, x_y, "x");
  outtextxy(y_x, y_y, "y");
  outtextxy(z_x, z_y, "z");
}

/* ---------------------------------------------------------------------- */

void DesenhaCurva3D(double (*f1)(double), double (*f2)(double),
     double (*f3)(double), double tmin, double tmax, double incr, int cor){

/* Desenha a curva (f1(t), f2(t), f3(t)) com t variando de tmin a tmax */

  double t, X, Y;

  for (t = tmin; t <= tmax; t += incr) {
    ProjetaPonto(f1(t), f2(t), f3(t), &X, &Y);
    putpixel(A + X*TAM, B - Y*TAM, cor);
  }
}

/* ---------------------------------------------------------------------- */

void DesenhaPonto3D(double X, double Y, double Z, int cor) {

/* Desenha o ponto (X, Y, Z) */

  double proj_X, proj_Y;

  ProjetaPonto(X, Y, Z, &proj_X, &proj_Y);
  putpixel(A + proj_X*TAM, B - proj_Y*TAM, cor);

}

/* ---------------------------------------------------------------------- */

void MarcaPonto3D(double X, double Y, double Z, int cor) {

/* Desenha o ponto (X, Y, Z) no formato de um "x" */

  double proj_X, proj_Y;

  ProjetaPonto(X, Y, Z, &proj_X, &proj_Y);
  putpixel(A + proj_X*TAM, B - proj_Y*TAM, cor);
  putpixel(A + proj_X*TAM + 1, B - proj_Y*TAM + 1, cor);
  putpixel(A + proj_X*TAM + 1, B - proj_Y*TAM - 1, cor);
  putpixel(A + proj_X*TAM - 1, B - proj_Y*TAM + 1, cor);
  putpixel(A + proj_X*TAM - 1, B - proj_Y*TAM - 1, cor);
  putpixel(A + proj_X*TAM + 2, B - proj_Y*TAM + 2, cor);
  putpixel(A + proj_X*TAM + 2, B - proj_Y*TAM - 2, cor);
  putpixel(A + proj_X*TAM - 2, B - proj_Y*TAM + 2, cor);
  putpixel(A + proj_X*TAM - 2, B - proj_Y*TAM - 2, cor);
}

#endif

/* ---------------------------------------------------------------------- */

/*** FIM DE GRAF.C ***/

