/*

    .oooo.     .oooo.        88       .o        88  .ooooo.    ooooooooo
  .dP""Y88b  .dP""Y88b      .8'     .d88       .8' 888' `Y88. d"""""""8'
	]8P'       ]8P'    .8'    .d'888      .8'  888    888       .8'
      .d8P'      <88b.    .8'   .d'  888     .8'    `Vbood888      .8'
    .dP'          `88b.  .8'    88ooo888oo  .8'          888'     .8'
  .oP     .o o.   .88P  .8'          888   .8'         .88P'     .8'
  8888888888 `8bd88P'   88          o888o  88        .oP'       .8'


     []-------------------------------------------------------------[]
     |	                     VETORES.C                               |
     |				                                     |
     |     Funcoes, macros, constantes para o calculo de vetores     |
     |	      e resolucao de sistemas lineares 2x2, 3x3 e 2x3        |
     |						                     |
     |           por Lenimar N. Andrade, maio/96-abril/97            |
     []-------------------------------------------------------------[]
*/

#include <math.h>
#include <stdio.h>

/* Quadrado de x */
#define SQR(x)  ((x)*(x))

/* Produto vetorial (v1, v2, v3) = (a1, a2, a3) x (b1, b2, b3) */
#define PROD_VET(v1,v2,v3,a1,a2,a3,b1,b2,b3)   v1=(a2)*(b3)-(a3)*(b2); \
		       v2=(a3)*(b1)-(a1)*(b3); v3=(a1)*(b2)-(a2)*(b1)

/* Norma do vetor (v1, v2, v3) */
#define NORMA(v1, v2, v3)       (sqrt((v1)*(v1)+(v2)*(v2)+(v3)*(v3)))

/* Distancia entre os pontos (x1, y1, z1) e (x2, y2, z2) */
#define DISTANCIA(x1,y1,z1,x2,y2,z2) sqrt(SQR((x1)-(x2)) + SQR((y1)-(y2)) \
							 + SQR((z1)-(z2)))

/* Determinante da matriz {{a, b}, {c, d}} */
#define Det2x2(a, b, c, d)  ((a)*(d) - (b)*(c))

/* Determinante da matriz {{a, b, c}, {d, e, f}, {g, h, i}} */
#define Det3x3(a,b,c,d,e,f,g,h,i)  ((a)*(e)*(i)+(c)*(d)*(h)+(b)*(f)*(g)\
				   -(c)*(e)*(g)-(a)*(f)*(h)-(b)*(d)*(i))
/* "Infinito" */
#define INF 3.4e38

/* Numero maximo de iteracoes permitido */
#define MAX_ITERACOES 100

/* f'(a) = derivada de f no ponto a */
#define DER(f,a,h)  ((f(a+h) - f(a-h))/(2.0*h))

/* f"(a) = derivada segunda de f no ponto a */
#define DER2(f,a,h) ((-2.0*f(a) + f(a-2.0*h) + f(a+2.0*h))/(4.0*h*h))

/* f'''(a) = derivada terceira de f no ponto a */
#define DER3(f,a,h) ((-f(a-3*h) + 3*f(a-h) - 3*f(a+h) + f(a+3*h))/(8.0*h*h*h))

/* Derivada parcial de f com relacao `a primeira variavel */
#define DIF1(f,a,b,h)  ((f(a+h,b) - f(a-h,b))/(2.0*h))

/* Derivada parcial de f com relacao `a segunda variavel */
#define DIF2(f,a,b,h)  ((f(a,b+h) - f(a,b-h))/(2.0*h))

/* Estruturas usadas ao longo de todo o programa */
typedef struct { double x, y, z; } vetor;       /* Vetor (x, y, z) no R3   */
typedef struct { double x, y, z; } ponto;       /* Ponto (x, y, z) no R3   */
typedef struct { double u, v; } ponto_dominio;  /* Ponto (u, v) no dominio */

/* Variaveis globais */
double h_erro = 1e-6,   /* Usado no calculo aproximado das derivadas    */
       z_erro = 1e-6;   /* Se |x| < z_erro, entao x sera' considerado 0 */

/* ----------------------------------------------------------------------- */

/* int ResolveSistema2x2(double A, double B, double M,
		      double C, double D, double N,
		      double *X, double *Y) {

/* Resolucao do sistema linear determinado   Ax + By = M
					     Cx + Dy = N
   Devolve 0 somente se houver erro na resolucao  */

  double det;

  det = Det2x2(A, B, C, D);
  if (fabs(det) < z_erro) return 0;
  *X = Det2x2(M, B, N, D)/det;
  *Y = Det2x2(A, M, C, N)/det;
  return 1;
} */

/* ----------------------------------------------------------------------- */

int ResolveSistema3x3(double A, double B, double C, double M,
		      double D, double E, double F, double N,
		      double G, double H, double I, double P,
		      double *X, double *Y, double *Z) {

/* Resolucao do sistema linear determinado
			 Ax + By + Cz = M
			 Dx + Ey + Fz = N
			 Gx + Hy + Iz = P
   Devolve 0 somente se a solucao unica nao puder ser encontrada. */

  double det;

  det = Det3x3(A, B, C, D, E, F, G, H, I);
  if (fabs(det) < z_erro) return 0;
  *X = Det3x3(M, B, C, N, E, F, P, H, I)/det;
  *Y = Det3x3(A, M, C, D, N, F, G, P, I)/det;
  *Z = Det3x3(A, B, M, D, E, N, G, H, P)/det;
  return 1;
}

/* ------------------------------------------------------------------------ */

/* int ResolveSistema2x3(double A, double B, double C, double M,
		      double E, double F, double G, double N,
		      double *X1, double *Y1, double *Z1,
		      double *X2, double *Y2, double *Z2) {

/* Determina 2 solucoes L.I. do sistema  Ax + By + Cz = M
					 Ex + Fy + Gz = N
   Retorna 0 somente se houver erro na resolucao. */

  if (fabs(A*F - B*E) > z_erro) {

    /* Resolve os sistemas Ax + By = M  e  Ax + By = M - C
			   Ex + Fy = N     Ex + Fy = N - G
			   z = 0           z = 1

    quando | A*F - B*E | > z_erro */

    if (!ResolveSistema2x2(A, B, M, E, F, N, X1, Y1)) return 0;
    *Z1 = 0;
    if (!ResolveSistema2x2(A, B, M - C, E, F, N - G, X2, Y2)) return 0;
    *Z2 = 1;
    return 1;
  }

  else if (fabs(A*G - C*E) > z_erro) {

    /* Resolve os sistemas Ax + Cz = M  e  Ax + Cz = M - B
			   Ex + Gz = N     Ex + Gz = N - F
			   y = 0           y = 1

    quando | A*G - C*E | > z_erro */

    if (!ResolveSistema2x2(A, C, M, E, G, N, X1, Z1)) return 0;
    *Y1 = 0;
    if (!ResolveSistema2x2(A, C, M - B, E, G, N - F, X2, Z2)) return 0;
   *Y2 = 1;
    return 1;

  } else if (fabs(B*G - C*F) > z_erro) {

    /* Resolve os sistemas By + Cz = M  e  By + Cz = M - A
			   Fy + Gz = N     Fy + Gz = N - E
			   x = 0           x = 1

    quando | B*G - C*F | > z_erro */

    if (!ResolveSistema2x2(B, C, M, F, G, N, Y1, Z1)) return 0;
    *X1 = 0;
    if (!ResolveSistema2x2(B, C, M - A, F, G, N - E, Y2, Z2)) return 0;
    *X2 = 1;
    return 1;
  }

  return 0;
}

/* ----------------------------------------------------------------------- */

int VetorNormal(vetor *N, double (*f1)(double,double),
		double (*f2)(double, double), double (*f3)(double, double),
		double u0, double v0) {

/* Calcula N, o vetor normal no  ponto  (u0, v0)  `a
   superficie f(u, v) = (f1(u, v), f2(u, v), f3(u, v)). Retorna 0
   se o vetor normal encontrado for nulo ou 1 em  caso contrario. */

  double a1, a2, a3, b1, b2, b3, norma_N;

  /* Calculo de 2 vetores tangentes `a superficie f */

  a1 = DIF1(f1, u0, v0, h_erro);
  a2 = DIF1(f2, u0, v0, h_erro);
  a3 = DIF1(f3, u0, v0, h_erro);

  b1 = DIF2(f1, u0, v0, h_erro);
  b2 = DIF2(f2, u0, v0, h_erro);
  b3 = DIF2(f3, u0, v0, h_erro);

  /* Vetor normal a f */
  PROD_VET(N->x, N->y, N->z, a1, a2, a3, b1, b2, b3);

  norma_N = NORMA(N->x, N->y, N->z);
  if (norma_N < z_erro) return 0;
  return 1;
}

/* ----------------------------------------------------------------------- */

int VetorTangente(vetor *T, double (*f1)(double, double),
		   double (*f2)(double, double), double (*f3)(double, double),
		   double u1, double v1,
		   double (*g1)(double, double), double (*g2)(double, double),
		   double (*g3)(double, double), double u2, double v2) {

/* Calcula T, o vetor unitario tangente `a curva intersecao das superficies
	       f(u, v) = (f1(u, v), f2(u, v), f3(u, v))  e
	       g(u, v) = (g1(u, v), g2(u, v), g3(u, v))
   Retorna 0 somente se houver erro no calculo do vetor tangente. */

  double a1, a2, a3, b1, b2, b3, c1, c2, c3, d1, d2, d3,
	F_n1, F_n2, F_n3, G_n1, G_n2, G_n3, norma_T;

  /* Verifica se o ponto pertence mesmo `a intersecao */
  if (DISTANCIA(f1(u1, v1), f2(u1, v1), f3(u1, v1),
		g1(u2, v2), g2(u2, v2), g3(u2, v2)) > z_erro) return 0;

  /* Calculo de 2 vetores tangentes `a superficie f */

  a1 = DIF1(f1, u1, v1, h_erro);
  a2 = DIF1(f2, u1, v1, h_erro);
  a3 = DIF1(f3, u1, v1, h_erro);

  b1 = DIF2(f1, u1, v1, h_erro);
  b2 = DIF2(f2, u1, v1, h_erro);
  b3 = DIF2(f3, u1, v1, h_erro);

  /* Calculo de 2 vetores tangentes `a superficie g */

  c1 = DIF1(g1, u2, v2, h_erro);
  c2 = DIF1(g2, u2, v2, h_erro);
  c3 = DIF1(g3, u2, v2, h_erro);

  d1 = DIF2(g1, u2, v2, h_erro);
  d2 = DIF2(g2, u2, v2, h_erro);
  d3 = DIF2(g3, u2, v2, h_erro);

  /* Vetor normal a f */
  PROD_VET(F_n1, F_n2, F_n3, a1, a2, a3, b1, b2, b3);

  /* Vetor normal a g */
  PROD_VET(G_n1, G_n2, G_n3, c1, c2, c3, d1, d2, d3);

  /* Vetor tangente `a intersecao */
  PROD_VET(T->x, T->y, T->z, F_n1, F_n2, F_n3, G_n1, G_n2, G_n3);

  norma_T = NORMA(T->x, T->y, T->z);
  if (norma_T < z_erro) return 0;

  T->x /= norma_T;
  T->y /= norma_T;
  T->z /= norma_T;

  return 1;
}       */

/* ----------------------------------------------------------------------- */

double Curvatura(double (*f1)(double), double (*f2)(double),
		 double (*f3)(double), double t) {

/* Calcula a curvatura de f(t) = (f1(t), f2(t), f3(t)) */

   vetor v, d1, d2;
   double a, b;

   d1.x = DER(f1, t, h_erro);
   d1.y = DER(f2, t, h_erro);
   d1.z = DER(f3, t, h_erro);

   d2.x = DER2(f1, t, h_erro);
   d2.y = DER2(f2, t, h_erro);
   d2.z = DER2(f3, t, h_erro);

   PROD_VET(v.x, v.y, v.z, d1.x, d1.y, d1.z, d2.x, d2.y, d2.z);
   a = NORMA(v.x, v.y, v.z);
   b = NORMA(d1.x, d1.y, d1.z);
   b = b*b*b;

   if (fabs(b) < z_erro) return INF; else return a/b;
}

/* ----------------------------------------------------------------------- */

void VetorTangenteACurva(vetor *v, double (*f1)(double), double (*f2)(double),
			double (*f3)(double), double t0) {

/* Calcula o vetor tangente `a curva (f1(t), f2(t), f3(t)) no ponto t = t0 */

  v->x = DER(f1, t0, h_erro);
  v->y = DER(f2, t0, h_erro);
  v->z = DER(f3, t0, h_erro);
}

/* ----------------------------------------------------------------------- */

double Dist(ponto P, double (*f1)(double), double (*f2)(double),
       double (*f3)(double), double a, double b, double incr, double *tmin) {

/* Calcula a distancia do ponto P `a curva f(t) = (f1(t), f2(t), f3(t)),
   a <= t <= b, usando incremento incr em t */

   double t, d, dmin = INF;

   t = a;
   do {
     d = DISTANCIA(P.x, P.y, P.z, f1(t), f2(t), f3(t));
     if (d < dmin) { dmin = d; *tmin = t; }
     t += incr;
   } while (t <= b);

   return dmin;
}

/* ----------------------------------------------------------------------- */

double Torcao(double (*f1)(double), double (*f2)(double),
		 double (*f3)(double), double t) {

/* Calcula a torcao de f(t) = (f1(t), f2(t), f3(t)) */

   vetor v, d1, d2, d3;
   double a, b;

   d1.x = DER(f1, t, h_erro);
   d1.y = DER(f2, t, h_erro);
   d1.z = DER(f3, t, h_erro);

   d2.x = DER2(f1, t, h_erro);
   d2.y = DER2(f2, t, h_erro);
   d2.z = DER2(f3, t, h_erro);

   d3.x = DER3(f1, t, h_erro);
   d3.y = DER3(f2, t, h_erro);
   d3.z = DER3(f3, t, h_erro);

   PROD_VET(v.x, v.y, v.z, d1.x, d1.y, d1.z, d2.x, d2.y, d2.z);

   a = v.x*d3.x + v.y*d3.y + v.z*d3.z;

   b = NORMA(v.x, v.y, v.z);
   b = b*b;

   if (fabs(b) < z_erro) return INF; else return a/b;
}


/* ----------------------------------------------------------------------- */

/*** FIM DE VETORES.C ***/
